import sys
import os
import struct
import intelhex

if len(sys.argv) < 2:
	sys.stderr.write('Usage: makecart.py PROGRAM.PRG\n')
	sys.exit(1)

fn = sys.argv[1]

# auto start?
if len(sys.argv) >= 3:
	auto = sys.argv[2]
else:
	auto = ""

# open wrapper and program file
f1 = open("cartload.abs", "rb") 
f2 = open(fn, "rb")

# read wrapper and program file into memory and concatenate
pd = f2.read()
fd = f1.read() + pd
f1.close()
f2.close()

# get file name and extension of program file and convert to 8.3 format
fn = os.path.basename(fn.upper())
(fn, ext) = os.path.splitext(fn)
fn = "%.8s%.4s" % (fn, ext)

# ... and check for validity
if (ext != '.TOS') and (ext != '.PRG'):
	sys.stderr.write('Only PRG or TOS file supported')
	sys.exit(1)

# check for max size
if (len(fd) > 64*1024):
	sys.stderr.write('Resulting image is larger than 64 kiB')
	sys.exit(1)
	
# patch in data field with program size and name
fd = bytearray(fd) 
struct.pack_into(">H13s", fd, 0x16, len(pd), fn)

# patch in auto start field if requested
if (auto == "-a"):
	struct.pack_into(">L", fd, 0x8, 0x08FA0026)

# and output binary file, extending to 64 kiB
fo = open("test.stc", "wb")
fo.write(fd)
fo.write(bytearray(65536 - len(fd))) # zero padding
fo.close()

# new IntelHex object
o = intelhex.IntelHex()
# pad to even length
if (len(fd) % 2) == 1:
	fd.append(0)
o.puts(0, fd)
o.write_hex_file("test.hex", write_start_addr=False)

if (auto == "-a"):
	sys.stderr.write('Auto-start cartridge image created: binary file test.stc and HEX file test.hex\n')
else:
	sys.stderr.write('Cartridge image created: binary file test.stc and HEX file test.hex\n')



